@@ -45,8 +45,8 @@
 /* Array of numbers to print, or NO_NUMBER */
 /* Where to put each tick/label on the y-axis */
 typedef struct {
-    double value;	/* Number to show, NO_NUMBER if just a tick */
-    double distance;	/* Distance of tick from min_y */
+    freq_t value;	/* Number to show, NO_NUMBER if just a tick */
+    unsigned short y;	/* screen coordinate of this tick */
 } tick_t;
 static tick_t *ticks = NULL;
 static int number_of_ticks = 0;	/* Number of labels allocated in this array */
@@ -67,8 +67,8 @@ static int decimal_places_to_print;
 
 /* Forward declarations */
 
-static int calculate_ticks(double min, double max, double distance, int log_scale);
-static int calculate_log_ticks(double min, double max, double distance);
+static int calculate_ticks(int log_scale);
+static int calculate_log_ticks(void);
 static void draw_frequency_axis(void);
 static void draw_note_names(void);
 
@@ -102,22 +102,22 @@ draw_time_axes(void)
 static void
 draw_frequency_axis()
 {
-    int tick_count = calculate_ticks(min_freq, max_freq, max_y - min_y, 1);
+    int tick_count = calculate_ticks(1);
     int i;
 
     if (min_x - 1 != frequency_axis_width - 1)
-    fprintf(stderr, "frequency axis with does not match min_x\n");
+    fprintf(stdout, "frequency axis with does not match min_x\n");
 
     gui_paint_rect(0, 0, min_x - 1, disp_height - 1, black);
 
     for (i=0; i < tick_count; i++) {
 	gui_lock();
-	gui_putpixel(min_x - 1, min_y + round(ticks[i].distance), green);
-	gui_putpixel(min_x - 2, min_y + round(ticks[i].distance), green);
+	gui_putpixel(min_x - 1, ticks[i].y, green);
+	gui_putpixel(min_x - 2, ticks[i].y, green);
 	gui_unlock();
 	if (ticks[i].value != NO_NUMBER) {
 	    draw_text(freq_to_string(ticks[i].value),
-		      min_x - 4, min_y + round(ticks[i].distance),
+		      min_x - 4, ticks[i].y,
 		      RIGHT, CENTER);
 	}
     }
@@ -145,13 +145,13 @@ draw_note_names()
 	for (note = ((octave == 0) ? 5 : 0);	/* from A0 */
 	     note <= ((octave == 9) ? 5 : 6);	/*   to A9 */
 	     note++) {
-	    double freq, half_a_pixel;
+	    freq_t freq, half_a_pixel;
 
 	    note_name[0] = note_names[note];
 	    freq = note_name_to_freq(note_name);
 	    if (freq == 0.0) {
 		/* Bad note name? Can't happen. */
-		fprintf(stderr, "Internal error in draw_note_names()\n");
+		fprintf(stdout, "Internal error in draw_note_names()\n");
 		continue;
 	    }
 	    half_a_pixel = sqrt(v_pixel_freq_ratio());
@@ -193,23 +193,29 @@ draw_note_names()
  */
 #define TARGET_DIVISIONS 3
 
-/* Helper function */
+/* Add a tick to the frequency axis
+ * k: which element of tick[] should we writing to?
+ * log_scale: Whether we're doing a linear or logarithmic section of the scale
+ * val: The frequency that this tick represents
+ * just_a_tick: Whether to display just a tick with no numeric frequency label
+ */
 static int
-add_tick(int k, double min, double max, double distance, int log_scale, double val, bool just_a_tick)
+add_tick(int k, int log_scale, freq_t val, bool just_a_tick)
 {
-    double range = max - min;
+    freq_t range = max_freq - min_freq;
 
     if (k >= number_of_ticks) {
 	ticks = Realloc(ticks, (k+1) * sizeof(*ticks));
 	number_of_ticks = k+1;
     }
 
-    if (DELTA_GE(val, min) && DELTA_LE(val, max)) {
+    if (DELTA_GE(val, min_freq) && DELTA_LE(val, max_freq)) {
 	ticks[k].value = just_a_tick ? NO_NUMBER : val;
-	ticks[k].distance =
-	    distance * (log_scale == 2
-		/*log*/	? (log(val) - log(min)) / (log(max) - log(min))
-		/*lin*/	: (val - min) / range);
+	ticks[k].y = min_y + round((max_y - min_y) *
+				   (log_scale == 2
+				    ? (log(val / min_freq)) /
+				      (log(max_freq / min_freq))
+				    : (val - min_freq) / range));
 	k++;
     }
     return k;
@@ -222,27 +228,27 @@ add_tick(int k, double min, double max, double distance, int log_scale, double v
  *   label the range with linear numbering but logarithmic spacing.
  */
 static int
-calculate_ticks(double min, double max, double distance, int log_scale)
+calculate_ticks(int log_scale)
 {
-    double stride;	/* Put numbered ticks at multiples of this */
-    double range = max - min;
+    freq_t stride;	/* Put numbered ticks at multiples of this */
+    freq_t range = max_freq - min_freq;
     int k;
-    double value;	/* Temporary */
+    freq_t value;	/* Temporary */
 
     if (log_scale == 1)
-	    return calculate_log_ticks(min, max, distance);
+	    return calculate_log_ticks();
 
     /* Linear version */
 
     /* Choose a step between successive axis labels so that one digit
-     * changes by 1, 2 or 5 amd that gives us at least the number of
+     * changes by 1, 2 or 5 and that gives us at least the number of
      * divisions (and numeric labels) that we would like to have.
      *
-     * We do this by starting "stride" at the lowest power of ten <= max,
+     * We do this by starting "stride" at the lowest power of ten <= max_freq,
      * which can give us at most 9 divisions (e.g. from 0 to 9999, step 1000)
      * Then try 5*this, 2*this and 1*this.
      */
-    stride = pow(10.0, floor(log10(max)));
+    stride = pow(10.0, floor(log10(max_freq)));
     do {
 	if (range / (stride * 5) >= TARGET_DIVISIONS) {
 	    stride *= 5;
@@ -261,20 +267,20 @@ calculate_ticks(double min, double max, double distance, int log_scale)
     if (decimal_places_to_print < 0)
 	    decimal_places_to_print = 0;
 
-    /* Now go from the first multiple of stride that's >= min to
-     * the last one that's <= max. */
+    /* Now go from the first multiple of stride that's >= min_freq to
+     * the last one that's <= max_freq. */
     k = 0;
-    value = ceil(min / stride) * stride;
+    value = ceil(min_freq / stride) * stride;
 
     /* Add the half-way tick before the first number if it's in range */
-    k = add_tick(k, min, max, distance, log_scale, value - stride / 2, TRUE);
+    k = add_tick(k, log_scale, value - stride / 2, TRUE);
 
-    while (DELTA_LE(value, max)) {
+    while (DELTA_LE(value, max_freq)) {
 	/* Add a tick next to each printed number */
-	k = add_tick(k, min, max, distance, log_scale, value, FALSE);
+	k = add_tick(k, log_scale, value, FALSE);
 
 	/* and at the half-way tick after the number if it's in range */
-	k = add_tick(k, min, max, distance, log_scale, value + stride/2, TRUE);
+	k = add_tick(k, log_scale, value + stride/2, TRUE);
 
 	value += stride;
     }
@@ -315,30 +321,30 @@ calculate_ticks(double min, double max, double distance, int log_scale)
  */
 
 /* Helper functions: add ticks and labels at start_value and all powers of ten
- * times it that are in the min-max range.
+ * times it that are in the min_freq..max_freq range.
  * This is used to plonk ticks at 1, 10, 100, 1000 then at 2, 20, 200, 2000
  * then at 5, 50, 500, 5000 and so on.
  */
 static int
-add_log_ticks(double min, double max, double distance,
-	      int k, double start_value, bool include_number)
+add_log_ticks(int k, freq_t start_value, bool include_number)
 {
-    double value;
+    freq_t value;
 
     /* Include the top and bottom rows if the tick for a frequency label
      * would round to that pixel row */
     double half_a_pixel = sqrt(v_pixel_freq_ratio());
 
-    for (value = start_value; DELTA_LE(value, max * half_a_pixel);
+    for (value = start_value; DELTA_LE(value, max_freq * half_a_pixel);
          value *= 10.0) {
 	if (k >= number_of_ticks) {
 	    ticks = Realloc(ticks, (k+1) * sizeof(*ticks));
 	    number_of_ticks = k+1;
 	}
-	if (DELTA_GE(value, min / half_a_pixel)) {
+	if (DELTA_GE(value, min_freq / half_a_pixel)) {
 	    ticks[k].value = include_number ? value : NO_NUMBER;
-	    ticks[k].distance = distance * (log(value) - log(min))
-					 / (log(max) - log(min));
+	    ticks[k].y = min_y + round((max_y - min_y)
+	    			       * (log(value) - log(min_freq))
+				       / (log(max_freq) - log(min_freq)));
 	    k++;
 	}
     }
@@ -346,40 +352,37 @@ add_log_ticks(double min, double max, double distance,
 }
 
 static int
-calculate_log_ticks(double min, double max, double distance)
+calculate_log_ticks(void)
 {
     int k = 0;	/* Number of ticks we have placed in "ticks" array */
-    double underpinning;	/* Largest power of ten that is <= min */
+    double underpinning;	/* Largest power of ten that is <= min_freq */
 
     /* If the interval is less than a decade, just apply the same
      * numbering-choosing scheme as used with linear axis, with the
      * ticks positioned logarithmically.
      */
-    if (max / min < 10.0)
-	    return calculate_ticks(min, max, distance, 2);
+    if (max_freq / min_freq < 10.0)
+	    return calculate_ticks(2);
 
     /* First hack: label the powers of ten. */
 
     /* Find largest power of ten that is <= minimum value */
-    underpinning = pow(10.0, floor(log10(min)));
+    underpinning = pow(10.0, floor(log10(min_freq)));
 
     /* Go powering up by 10 from there, numbering as we go. */
-    k = add_log_ticks(min, max, distance, k, underpinning, TRUE);
+    k = add_log_ticks(k, underpinning, TRUE);
 
     /* Do we have enough numbers? If so, add numberless ticks at 2 and 5 */
     /* No of labels is n. of divisions + 1 */
     if (k >= TARGET_DIVISIONS + 1) {
-	k = add_log_ticks(min, max, distance, k,
-			  underpinning * 2.0, FALSE);
-	k = add_log_ticks(min, max, distance, k,
-			  underpinning * 5.0, FALSE);
+	k = add_log_ticks(k, underpinning * 2.0, FALSE);
+	k = add_log_ticks(k, underpinning * 5.0, FALSE);
     } else {
 	int i;
 	/* Not enough numbers: add numbered ticks at 2 and 5 and
 	 * unnumbered ticks at all the rest */
 	for (i = 2; i <= 9; i++)
-	    k = add_log_ticks(min, max, distance, k,
-			      underpinning * (1.0 * i),
+	    k = add_log_ticks(k, underpinning * (1.0 * i),
 			      i == 2 || i == 5);
     }
 
@@ -424,7 +427,7 @@ draw_status_line(void)
     /* Left-justified with one blank column to its left */
     sprintf(s, "%.*g-", hz_prec(min_freq), min_freq);
     sprintf(s+strlen(s), "%.*gHz, ", hz_prec(max_freq), max_freq);
-    sprintf(s+strlen(s), "%.2g octaves", log(max_freq / min_freq) / log(2.0));
+    sprintf(s+strlen(s), "%.3g octaves", log(max_freq / min_freq) / log(2.0));
     draw_text(s, min_x + 1, max_y + 2, LEFT, BOTTOM);
 
     /* Centered */
@@ -521,35 +524,71 @@ draw_time_axis(void)
     gui_update_rect(min_x, 0, max_x, min_y - 1);
 }
 
-/* Show on the time axis which part of the audio file is displayed */
+/* Show on the time axis which part of the audio file is displayed.
+ *
+ * The width of the progress bar represents the whole piece from time 0
+ * to time audio_file_length() and the edges of the cursor represent the
+ * edges of what you can see on the screen.
+ */
 static void
 draw_time_axis_progress(void)
 {
-    double piece_ppsec = (max_x - min_x) / audio_file_length();
-    int x1 = floor(min_x + screen_column_to_start_time(min_x) * piece_ppsec);
-    int x2 = ceil(screen_column_to_start_time(max_x) * piece_ppsec + DELTA);
-    int width = round(((max_x - min_x + 1) / ppsec) / audio_file_length() * (max_x - min_x + 1));
-    int y1 = 0;
-    int y2 = bottom_margin - 1;
+    int x1, x2;	 /* Left and right box pixel positions */
+    int y1, y2;  /* bottom and top box pixe lpositions */
 
-    /* The above x1/x2 formulae make it correctly place the sides of the box
-     * at column min_x or max_x when start or end of piece is at column min_x
-     * or max_x but this makes the box's width change so it crawls like a
-     * caterpillar. We obtain constant box width as well as start/end alignment
+    /* How much time does the graphic show? */
+    unsigned graph_width = max_x - min_x + 1;
+    secs_t graph_time = graph_width / ppsec;
+
+    /* Cursor size measured from left edge of left pixel to right of right */
+    unsigned box_width = graph_width * (graph_time / audio_file_length());
+
+    /* It's no good calculating the left and right positions separately
+     * because that makes the left and right edges update independently
+     * making the box's width change and the box crawl like a caterpillar.
+     * We obtain constant box width as well as start/end alignment
      * by aligning a fixed-width box's left edge if it's on the left half of
      * the screen, and aligning the right edge if it's on the right.
      * No one will notice a possible single-pixel position irregularity
      * as the box slides past the date line.
      */
-    if ((x1 + x2) / 2 < (max_x + min_x) / 2)
-	x2 = x1 + width;
-    else
-	x1 = x2 - width;
+    if (disp_time < audio_file_length() / 2) {
+	x1 = min_x + floor(graph_width *
+		     screen_column_to_start_time(min_x) / audio_file_length());
+	x2 = x1 + (box_width - 1);
+    } else {
+	x2 = min_x + floor(graph_width *
+		     screen_column_to_start_time(max_x) / audio_file_length());
+        x1 = x2 - (box_width - 1);
+    }
 
+    y1 = 0; y2 = time_axis_height - 1;
+
+    /* Don't spill over into the frequency axes,
+     * and if the left/right edge is outside the graph, don't show it.
+     */
+#ifndef min
+# define min(a,b) ((a)<=(b)?(a):(b))
+#endif
+#ifndef max
+# define max(a,b) ((a)>=(b)?(a):(b))
+#endif
     gui_lock();
-    gui_draw_line(x1, y1, x2, y1, white);
-    gui_draw_line(x2, y1, x2, y2, white);
-    gui_draw_line(x2, y2, x1, y2, white);
-    gui_draw_line(x1, y2, x1, y1, white);
+    /* top */    gui_draw_line(max(x1, min_x), y2, min(x2, max_x), y2, white);
+    /* bottom */ gui_draw_line(max(x1, min_x), y1, min(x2, max_x), y1, white);
+    /* left */   if (x1 >= min_x) gui_draw_line(x1, y1, x1, y2, white);
+    /* right */  if (x2 <= max_x) gui_draw_line(x2, y1, x2, y2, white);
     gui_unlock();
 }
+
+void
+show_title(bool onoff)
+{
+  if (!show_songbar && onoff) {
+     /* Add the song's name bar under the time axis */
+    show_songbar = 1;
+  } else {
+    /* Remove the songbar */
+    show_songbar = 0;
+  }
+}
