@@ -73,6 +73,12 @@ k_next(key_t key)
     gui_quit_main_loop();
 }
 
+static void
+k_minimize(key_t key)
+{
+    gui_minimize();
+}
+
 static void
 k_space(key_t key)	/* Play/Pause/Next_file/Quit */
 {
@@ -223,7 +229,7 @@ k_set_window(key_t key)
     case KEY_B: new_fn = BLACKMAN;	break;
     case KEY_D: new_fn = DOLPH;		break;
     default:
-	fprintf(stderr, "Internal error: Impossible window key %d\n", key);
+	fprintf(stdout, "Internal error: Impossible window key %d\n", key);
 	return;
     }
 
@@ -421,6 +427,9 @@ k_print_params(key_t key)
 #if HAVE_LIBSNDFILE
 	   af->opened_with == libsndfile ? "libsndfile" :
 #endif
+#if HAVE_LIBSOX
+	   af->opened_with == libsox ? "libsox" :
+#endif
 #if HAVE_LIBAV
 	   af->opened_with == libav ? "libav" :
 #endif
@@ -613,7 +622,7 @@ k_beats_per_bar(key_t key)
 		case KEY_F11:	set_beats_per_bar(11);	break;
 		case KEY_F12:	set_beats_per_bar(12);	break;
     default:
-	fprintf(stderr, "Internal error: Impossible beats-per-bar key %d\n", key);
+	fprintf(stdout, "Internal error: Impossible beats-per-bar key %d\n", key);
 	return;
     }
 }
@@ -646,7 +655,7 @@ static key_fn key_fns[];
 static void
 k_bad(key_t key)
 {
-    fprintf(stderr, "%s%s%s doesn't do anything\n",
+    fprintf(stdout, "%s%s%s doesn't do anything\n",
 	Ctrl  ? "Ctrl-"  : "",
 	Shift ? "Shift-" : "",
 	key_fns[key].name);
@@ -657,6 +666,7 @@ static key_fn key_fns[] = {
     /* KEY	Name    plain		Shift		Ctrl		ShiftCtrl */
     { KEY_NONE,	"None", k_none,		k_none,		k_none,		k_none },
     { KEY_Q,	"Q",	k_quit,		k_quit,		k_quit,		k_quit },
+    { KEY_U,	"U",	k_bad,		k_minimize,	k_bad,		k_bad },
     { KEY_C,	"C",    k_cycle_colors,	k_bad,		k_bad,		k_bad },
     { KEY_I,	"I",    k_brightness,	k_brightness,	k_brightness,	k_brightness },
     { KEY_Z,	"Z",    k_contrast,	k_contrast,	k_contrast,	k_contrast },
@@ -732,7 +742,7 @@ do_key(key_t key)
 
     for (i=0; i < N_KEYS; i++) {
 	if (key_fns[i].key != i) {
-	    fprintf(stderr, "Key function table is skewed at element %d\n", i);
+	    fprintf(stdout, "Key function table is skewed at element %d\n", i);
 	    return;
 	}
 	if (key_fns[i].key == key) {
@@ -742,5 +752,5 @@ do_key(key_t key)
 	    if (Shift && Ctrl)	{ (*key_fns[i].shiftctrl)(key);	return; }
 	}
     }
-    fprintf(stderr, "Internal error: Impossible key value %d\n", key);
+    fprintf(stdout, "Internal error: Impossible key value %d\n", key);
 }
