@@ -32,6 +32,10 @@
 #include "timer.h"
 #include "ui.h"
 
+#if HAVE_SETJMP
+# include <setjmp.h>
+#endif
+
 /* Libraries' header files. See config.h for working combinations of defines */
 
 #if HAVE_LIBPNG
@@ -39,6 +43,10 @@
 #endif
 #include <errno.h>
 
+#if ITIMER_TIMER
+bool scroll_pending = FALSE;
+#endif
+
 #if ECORE_TIMER || EVAS_VIDEO || ECORE_MAIN
 #include <Ecore.h>
 #include <Ecore_Evas.h>
@@ -133,7 +141,7 @@ gui_init(char *filename)
 
 #if SDL_VIDEO
     if (SDL_InitSubSystem(SDL_INIT_VIDEO) != 0) {
-	fprintf(stderr, "Couldn't initialize SDL video subsystem: %s.\n", SDL_GetError());
+	fprintf(stdout, "Couldn't initialize SDL video subsystem: %s.\n", SDL_GetError());
 	goto end_sdl;
     }
 
@@ -172,20 +180,22 @@ gui_init(char *filename)
 			      (fullscreen_mode ? SDL_WINDOW_FULLSCREEN : 0));
 				 /* | SDL_WINDOW_RESIZABLE one day... */
     if (sdl_window == NULL) {
-	fprintf(stderr, "Couldn't create SDL window: %s\n", SDL_GetError());
+	fprintf(stdout, "Couldn't create SDL window: %s\n", SDL_GetError());
 	exit(1);
     }
 
-    sdl_renderer = SDL_CreateRenderer(sdl_window, -1, 0);
+    sdl_renderer = SDL_CreateRenderer(sdl_window, -1,
+				      software_renderer ? SDL_RENDERER_SOFTWARE
+							: 0);
 	/* maybe SDL_RENDERER_PRESENTVSYNC */
     if (sdl_renderer == NULL) {
-	fprintf(stderr, "Couldn't create SDL renderer: %s\n", SDL_GetError());
+	fprintf(stdout, "Couldn't create SDL renderer: %s\n", SDL_GetError());
 	exit(1);
     }
 
     surface = SDL_GetWindowSurface(sdl_window);
     if (surface == NULL) {
-	fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+	fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 		SDL_GetError());
 	exit(1);
     }
@@ -201,7 +211,7 @@ gui_init(char *filename)
 
     /* Clear the image buffer to the background color */
     if (SDL_FillRect(surface, NULL, background) != 0) {
-	fprintf(stderr, "Couldn't fill surface with background color: %s\n",
+	fprintf(stdout, "Couldn't fill surface with background color: %s\n",
 		SDL_GetError());
 	exit(1);
     }
@@ -216,7 +226,7 @@ end_sdl:
     /* Initialize the graphics subsystem */
     if (!ecore_evas_init() ||
 	!(ee = ecore_evas_new(NULL, 0, 0, disp_width, disp_height, NULL))) {
-	fputs("Cannot initialize EFL graphics subsystem.\n", stderr);
+	fputs("Cannot initialize EFL graphics subsystem.\n", stdout);
 	goto end_efl;
     }
     ecore_evas_callback_delete_request_set(ee, quit_ecore_evas);
@@ -272,7 +282,7 @@ end_sdl:
 end_efl:
 #endif
 
-    fprintf(stderr, "No video driver managed to initialise.\n");
+    fprintf(stdout, "No video driver managed to initialise.\n");
     exit(1);
 
 got_video_driver:
@@ -292,9 +302,9 @@ got_video_driver:
 # endif
 	if (!em) {
 # if EMOTION_AUDIO
-	    fputs("Couldn't initialize Emotion audio.\n", stderr);
+	    fputs("Couldn't initialize Emotion audio.\n", stdout);
 # else
-	    fputs("Couldn't initialize Evas graphics.\n", stderr);
+	    fputs("Couldn't initialize Evas graphics.\n", stdout);
 # endif
 	    exit(1);
 	}
@@ -304,8 +314,7 @@ got_video_driver:
     case video_sdl:
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -346,12 +355,25 @@ gui_deinit()
 	/* SDL_Quit(); is called by atexit() if we are quitting */
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
 
+void
+gui_minimize()
+{
+    switch (video_driver) {
+#if EVAS_VIDEO
+    case video_efl: /* TODO */ break;
+#endif
+#if SDL_VIDEO
+    case video_sdl: SDL_MinimizeWindow(sdl_window); break;
+#endif
+    case video_none: break;
+    }
+}
+
 /* Tell the video subsystem to update the display from the pixel data */
 void
 gui_update_display()
@@ -372,8 +394,7 @@ gui_update_display()
 	SDL_UpdateWindowSurface(sdl_window);
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -414,12 +435,12 @@ gui_update_rect(int from_x, int from_y, int to_x, int to_y)
 	rect.h = height;
 
 	if (SDL_UpdateWindowSurfaceRects(sdl_window, &rect, 1) != 0) {
-	    fprintf(stderr, "SDL_UpdateWindowSurfaceRects failed: %s\n",
+	    fprintf(stdout, "SDL_UpdateWindowSurfaceRects failed: %s\n",
 		    SDL_GetError());
 	}
 	break;
 #endif
-    default:
+    case video_none:
 	break;
     }
 }
@@ -537,7 +558,7 @@ gui_main()
 		break;
 
 	    default:
-		fprintf(stderr, "Unknown SDL_USEREVENT code %d\n",
+		fprintf(stdout, "Unknown SDL_USEREVENT code %d\n",
 			event.user.code);
 		break;
 	    }
@@ -548,8 +569,7 @@ gui_main()
 	}
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -628,13 +648,12 @@ gui_quit_main_loop(void)
     case video_sdl:
     event.type = SDL_QUIT;
 	if (SDL_PushEvent(&event) != SDL_PUSHEVENT_SUCCESS) {
-	    fprintf(stderr, "sdl_main_loop_quit event push failed: %s\n", SDL_GetError());
+	    fprintf(stdout, "sdl_main_loop_quit event push failed: %s\n", SDL_GetError());
 	    exit(1);
 	}
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -665,8 +684,7 @@ gui_fullscreen()
 	repaint_display(FALSE);
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -728,20 +746,19 @@ gui_h_scroll_by(int scroll_by)
 	if (scroll_by != 0) {
 	    SDL_Surface *surface = SDL_GetWindowSurface(sdl_window);
 	    if (surface == NULL) {
-		fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+		fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 			SDL_GetError());
 		exit(1);
 	    }
 	    from.y = to.y = (disp_height - 1) - max_y;
 	    from.h = max_y - min_y + 1;
 	    if ((err = SDL_BlitSurface(surface, &from, surface, &to)) != 0) {
-		fprintf(stderr, "SDL Blit failed with value %d.\n", err);
+		fprintf(stdout, "SDL Blit failed with value %d.\n", err);
 	    }
 	}
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -819,18 +836,17 @@ gui_v_scroll_by(int scroll_by)
 	if (scroll_by != 0) {
 	    SDL_Surface *surface = SDL_GetWindowSurface(sdl_window);
 	    if (surface == NULL) {
-		fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+		fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 			SDL_GetError());
 		exit(1);
 	    }
 	    if ((err = SDL_BlitSurface(surface, &from, surface, &to)) != 0) {
-		fprintf(stderr, "SDL Blit failed with value %d.\n", err);
+		fprintf(stdout, "SDL Blit failed with value %d.\n", err);
 	    }
 	}
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -865,7 +881,7 @@ gui_paint_rect(int from_x, int from_y, int to_x, int to_y, color_t color)
     case video_sdl:
 	surface = SDL_GetWindowSurface(sdl_window);
 	if (surface == NULL) {
-	    fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+	    fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 		    SDL_GetError());
 	    exit(1);
 	}
@@ -876,8 +892,7 @@ gui_paint_rect(int from_x, int from_y, int to_x, int to_y, color_t color)
 	SDL_FillRect(surface, &rect, color);
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -925,12 +940,12 @@ gui_lock()
     case video_sdl:
 	sdl_surface = SDL_GetWindowSurface(sdl_window);
 	if (sdl_surface == NULL) {
-	    fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+	    fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 		    SDL_GetError());
 	    exit(1);
 	}
 	if (SDL_MUSTLOCK(sdl_surface) && SDL_LockSurface(sdl_surface) != 0 ) {
-	    fprintf(stderr, "Can't lock surface: %s\n", SDL_GetError());
+	    fprintf(stdout, "Can't lock surface: %s\n", SDL_GetError());
 	}
 	break;
 #endif
@@ -945,7 +960,8 @@ gui_unlock()
     switch (video_driver) {
 #if SDL_VIDEO
     case video_sdl:
-	if (SDL_MUSTLOCK(sdl_surface)) SDL_UnlockSurface(sdl_surface);
+	if (sdl_surface && SDL_MUSTLOCK(sdl_surface))
+            SDL_UnlockSurface(sdl_surface);
 	sdl_surface = NULL;
 	break;
 #endif
@@ -972,17 +988,17 @@ RGB_to_color(primary_t red, primary_t green, primary_t blue)
     case video_sdl:
 	if (sdl_surface == NULL) {
 	    /* Has happened once, in call paint_column->colormap->RGB_to_color */
-	    fprintf(stderr, "Internal error: Calls to RGB_to_color() must be wrapped with gui_lock()/gui_unlock()\n");
+	    fprintf(stdout, "Internal error: Calls to RGB_to_color() must be wrapped with gui_lock()/gui_unlock()\n");
 	    /* Mystery. Try anyway. */
 	    sdl_surface = SDL_GetWindowSurface(sdl_window);
 	    if (sdl_surface == NULL) abort();
 	}
 	return SDL_MapRGB(sdl_surface->format, red, green, blue);
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
+    return no_color;
 }
 
 /* Calls to this should be bracketed by gui_lock() and gui_unlock(). */
@@ -1006,8 +1022,8 @@ gui_putpixel(int x, int y, color_t color)
 #if SDL_VIDEO
     case video_sdl:
 	if (sdl_surface == NULL) {
-	    fprintf(stderr, "Internal error: Calls to gui_putpixel() must be wrapped with gui_lock()/gui_unlock()\n");
-	    abort();
+	    fprintf(stdout, "Internal error: Calls to gui_putpixel() must be wrapped with gui_lock()/gui_unlock()\n");
+	    return;
 	}
     /* Macro derived from http://sdl.beuc.net/sdl.wiki/Pixel_Access's putpixel() */
     #define putpixel(x, y, pixel) \
@@ -1017,8 +1033,7 @@ gui_putpixel(int x, int y, color_t color)
 	putpixel(x, (disp_height-1) - y, color);
 	break;
 #endif
-    default:
-	abort();
+    case video_none:
 	break;
     }
 }
@@ -1031,7 +1046,7 @@ bool
 gui_output_png_file(const char *filename)
 {
 #if !HAVE_LIBPNG
-    fprintf(stderr, "Screen dumping requires compilation with libpng.\n");
+    fprintf(stdout, "Screen dumping requires compilation with libpng.\n");
     return FALSE;
 #else
     png_structp png_ptr;
@@ -1047,29 +1062,31 @@ gui_output_png_file(const char *filename)
 
     png_ptr  = png_create_write_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
     if (png_ptr == NULL) {
-	fprintf(stderr, "Can't create PNG write structure.\n");
+	fprintf(stdout, "Can't create PNG write structure.\n");
 	return FALSE;
     }
     if (png_width > png_get_user_width_max(png_ptr) ||
 	png_height > png_get_user_height_max(png_ptr)) {
-	fprintf(stderr, "Image is too large for libpng.\n");
+	fprintf(stdout, "Image is too large for libpng.\n");
 	png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
 	return FALSE;
     }
 
     png_info = png_create_info_struct(png_ptr);
     if (png_info == NULL) {
-	fprintf(stderr, "Can't create PNG info structure.\n");
+	fprintf(stdout, "Can't create PNG info structure.\n");
 	png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
 	return FALSE;
     }
 
+#if HAVE_SETJMP
     /* Set up handling of errors within libpng */
     if (setjmp(png_jmpbuf(png_ptr))) {
 	png_destroy_write_struct(&png_ptr, &png_info);
 	if (file != NULL) fclose(file);
 	return FALSE;
     }
+#endif
 
     png_rows = png_malloc(png_ptr, png_height * sizeof (png_bytep));
 
@@ -1078,7 +1095,7 @@ gui_output_png_file(const char *filename)
 
     file = fopen(filename, "wb");
     if (file == NULL) {
-	fprintf(stderr, "Can't open \"%s\": %s\n", filename, strerror(errno));
+	fprintf(stdout, "Can't open \"%s\": %s\n", filename, strerror(errno));
 	png_destroy_write_struct(&png_ptr, &png_info);
 	return FALSE;
     }
@@ -1093,7 +1110,7 @@ gui_output_png_file(const char *filename)
     case video_sdl:
 	surface = SDL_GetWindowSurface(sdl_window);
 	if (surface == NULL) {
-	    fprintf(stderr, "Couldn't get SDL Window Surface: %s\n",
+	    fprintf(stdout, "Couldn't get SDL Window Surface: %s\n",
 		    SDL_GetError());
 	    exit(1);
 	}
@@ -1110,8 +1127,8 @@ gui_output_png_file(const char *filename)
 		(&imagedata[imagestride * y]);
 	break;
 # endif
-    default:
-	abort();
+
+    case video_none:
 	break;
     }
 
