@@ -39,3 +39,16 @@ strdup(const char *s)
     return new;
 }
 #endif
+
+#if !HAVE_STRCHR
+char *
+strchr(const char *s, const char c)
+{
+    char *p;
+
+    for (p=s; *p; p++)
+        if (*p == c) return p;
+
+    return NULL;
+}
+#endif
